from typing import Optional, List

from pydantic import BaseModel, Field


class TheStageBaseResponse(BaseModel):
    message: Optional[str] = Field(None, alias='message')
    is_success: Optional[bool] = Field(None, alias='isSuccess')


# TODO: add Generic then back doing that
class TheStageBasePaginatedResponse(TheStageBaseResponse):
    fields_with_error: List[str] = Field(None, alias='fieldsWithError')



from thestage_core.services.config_provider.config_provider import ConfigProviderCore


class ConfigProvider(ConfigProviderCore):

    def __init__(
            self,
            project_path: str,
            auto_create: bool = True,
            only_global: bool = False,
    ):
        super(ConfigProvider, self).__init__(
            project_path=project_path,
            auto_create=auto_create,
            only_global=only_global,
        )



import os

THESTAGE_CONFIG_DIR = os.getenv('THESTAGE_CONFIG_DIR', '.thestage')
THESTAGE_CONFIG_FILE = os.getenv('THESTAGE_CONFIG_FILE', 'config.json')
THESTAGE_AUTH_TOKEN = os.getenv('THESTAGE_AUTH_TOKEN', None)
THESTAGE_LOGGING_FILE = os.getenv('THESTAGE_LOGGING_FILE', 'thestage.log')
#THESTAGE_API_URL = os.getenv('THESTAGE_API_URL', 'https://backend-staging.thestage.ai')



from enum import Enum


class PowerStatusEnumDto(str, Enum):
    ON: str = 'ON'
    OFF: str = 'OFF'
    POWERING_ON: str = 'POWERING_ON'
    POWERING_OFF: str = 'POWERING_OFF'
    REBOOTING: str = 'REBOOTING'
    UNKNOWN: str = 'UNKNOWN'



import datetime
from pathlib import Path
from typing import Optional, List, Tuple

import git
import typer
from git import Remote, Repo, GitCommandError, Commit, Head, RemoteReference
from thestage_core.services.filesystem_service import FileSystemServiceCore

from thestage.exceptions.git_access_exception import GitAccessException


class GitLocalClient:
    __base_name_remote: str = 'origin'
    __base_name_local: str = 'main'
    __git_ignore_tsr_line: str = '/.thestage/'

    __special_main_branches = ['main', 'master']

    __base_git_url: str = 'https://github.com/'

    def __init__(
            self,
            file_system_service: FileSystemServiceCore,
    ):
        self.__file_system_service = file_system_service

    def __get_repo(self, path: str) -> Repo:
        return git.Repo(path)

    def is_present_local_git(self, path: str) -> bool:
        git_path = self.__file_system_service.get_path(path)
        if not git_path.exists():
            return False

        git_path = git_path.joinpath('.git')
        if not git_path.exists():
            return False

        result = git.repo.base.is_git_dir(git_path)
        return result

    def get_remote(self, path: str) -> Optional[List[Remote]]:
        is_git_repo = self.is_present_local_git(path=path)
        if is_git_repo:
            repo = git.Repo(path)
            remotes: Optional[List[Remote]] = list(repo.remotes) if repo.remotes else []
            return remotes
        return None

    def has_remote(self, path: str) -> bool:
        remotes: Optional[List[Remote]] = self.get_remote(path)
        return True if remotes is not None and len(remotes) > 0 else False

    def has_changes(self, path: str) -> bool:
        repo = self.__get_repo(path=path)
        is_dirty = repo.is_dirty()
        if is_dirty:
            return True
        return False

    def init_repository(
            self,
            path: str,
    ) -> Optional[Repo]:

        repo = git.Repo.init(path)
        if repo:
            # default git name master, rename to main - sync wih github
            repo.git.branch("-M", self.__base_name_local)
        return repo

    def add_remote_to_repo(
            self,
            path: str,
            remote_url: str,
            remote_name: str,
    ) -> bool:
        repo = self.__get_repo(path=path)
        remotes: List[Remote] = repo.remotes
        not_present = True
        if remotes:
            item = list(filter(lambda x: x.name == remote_name, remotes))
            if len(item) > 0:
                not_present = False

        if not_present:
            remote: Remote = repo.create_remote(
                name=self.__base_name_remote,
                url=remote_url,
            )
            if remote:
                return True
            else:
                return False
        else:
            return True

    def git_fetch(self, path: str):
        repo = self.__get_repo(path=path)
        if repo:
            remote: Remote = repo.remote(self.__base_name_remote)
            if remote:
                try:
                    remote.fetch()
                except GitCommandError as base_ex:
                    msg = base_ex.stderr
                    if msg and 'fatal: Could not read from remote repository' in msg:
                        raise GitAccessException(
                            message='You dont have access to repository, or repository not found.',
                            url=self.build_http_repo_url(git_path=remote.url),
                            dop_message=msg,
                        )
                    else:
                        raise base_ex

    def git_pull(self, path: str, branch: Optional[str] = None):
        repo = self.__get_repo(path=path)
        if repo:
            local_branch = self.__base_name_local
            if branch:
                if self.__base_name_remote in branch:
                    local_branch = branch.replace(f"{self.__base_name_remote}/", '').strip()
                else:
                    local_branch = branch

            repo.git.pull(self.__base_name_remote, local_branch)

    def find_main_branches(self, path: str) -> Tuple[Optional[str], bool]:
        repo = self.__get_repo(path=path)
        if repo:
            for ref in repo.git.branch('-a').split('\n'):
                for main_branch in self.__special_main_branches:
                    if main_branch in ref.split('/'):
                        if 'remotes/' in ref:
                            return ref.replace('remotes/', '').strip(), True
                        else:
                            return ref.strip(), False

        return None, False

    def git_checkout(self, path: str, branch: Optional[str] = None):
        repo = self.__get_repo(path=path)
        if repo:
            branch_name = repo.active_branch.name if repo.active_branch else self.__base_name_local
            if branch:
                branch_name = branch

            repo.git.checkout('-b', branch_name.strip())

    def add_new_branch(self, path: str, new_branch_name: Optional[str] = None,) -> Optional[Head]:
        # TODO: check and changes on future, fucking branches
        repo = self.__get_repo(path=path)
        if repo:
            branch = self.find_main_branches(path=path)
            if branch:
                return branch
            else:
                branch_name = new_branch_name if new_branch_name else self.__base_name_local

            origin = repo.remote(self.__base_name_remote)
            if origin:
                repo.create_head(self.__base_name_local, origin.refs.main)  # create local branch "master" from remote "master"
                repo.heads.main.set_tracking_branch(origin.refs.main)  # set local "master" to track remote "master
                repo.heads.master.checkout()

            #if new_branch_name:
                branch = repo.create_head(branch_name)
            #else:
                # if we want added special branch - main - first branch
                #origin = repo.remote(self.__base_name_remote)
            #    repo.head.reference = repo.create_head(branch_name)
                # Create new remote ref and set it to track.
            #    rem_ref = RemoteReference(repo, f"refs/remotes/{self.__base_name_remote}/{branch_name}")
            #    repo.head.reference.set_tracking_branch(rem_ref)

            return branch
        return None

    def build_http_repo_url(self, git_path: str) -> str:
        start_path_pos = git_path.find(":")
        pre_url = git_path[start_path_pos+1:]
        url = pre_url.replace('.git', '')
        return self.__base_git_url + url

    def clone(self, url: str, path: str, ) -> Optional[Repo]:
        try:
            return git.Repo.clone_from(url=url, to_path=path)
        except GitCommandError as base_ex:
            msg = base_ex.stderr
            if msg and 'Repository not found' in msg and 'correct access rights' in msg:
                raise GitAccessException(
                    message='You dont have access to repository, or repository not found.',
                    url=self.build_http_repo_url(git_path=url),
                    dop_message=msg,
                )
            else:
                raise base_ex

    def commit_local_changes(
            self,
            path: str,
            with_push: bool = False,
            author: Optional[str] = None,
            name: Optional[str] = None,
    ) -> Optional[str]:
        repo = self.__get_repo(path=path)
        commit_name = name if name else f"Auto commit {str(datetime.datetime.now().date())}"
        commit = repo.git.commit('--all', '--allow-empty', '-m', commit_name, )
        if with_push:
            self.push_changes(path=path, branch_name=repo.active_branch.name)
        return commit

    def push_changes(
            self,
            path: str,
            branch_name: str,
    ):
        repo = self.__get_repo(path=path)
        origin = repo.remote(self.__base_name_remote)
        if origin:
            repo.git.push('-u', origin.name, branch_name)

    def get_latest_commit(self, path: str, ) -> Optional[Commit]:
        repo = self.__get_repo(path=path)
        if repo:
            return repo.head.commit
        else:
            return None

    def _get_gitignore_path(self, path: str) -> Path:
        git_path = self.__file_system_service.get_path(path)
        return git_path.joinpath('.gitignore')

    def git_add_by_path(self, repo_path: str, file_path: str):
        repo = self.__get_repo(path=repo_path)
        if repo:
            repo.index.add(items=[file_path])

    def add_to_gitignore(self, path: str):

        gitignore_path = self._get_gitignore_path(path=path)
        if not gitignore_path.exists():
            self.__file_system_service.create_if_not_exists_file(gitignore_path)
            self.git_add_by_path(repo_path=path, file_path=str(gitignore_path))

        is_present_tsr = self.__file_system_service.find_in_text_file(file=str(gitignore_path),
                                                                      find=self.__git_ignore_tsr_line)
        if not is_present_tsr:
            self.__file_system_service.add_line_to_text_file(file=str(gitignore_path),
                                                             new_line=self.__git_ignore_tsr_line)



import os
from typing import Any, Callable

import typer
from click.exceptions import Exit
from git import GitCommandError

from thestage.config import THESTAGE_API_URL
from thestage.exceptions.remote_server_exception import RemoteServerException
from thestage.i18n.translation import __
from thestage.exceptions.git_access_exception import GitAccessException
from thestage.exceptions.auth_exception import AuthException
from thestage.exceptions.business_logic_exception import BusinessLogicException
from thestage.exceptions.config_exception import ConfigException
from thestage.exceptions.file_system_exception import FileSystemException
from thestage.exceptions.http_error_exception import HttpClientException
from thestage.helpers.logger.app_logger import app_logger


def error_handler() -> Callable:
    def wrap(f):
        def wrapper(*args: Any, **kwargs: Any) -> Any:
            try:
                result = f(*args, **kwargs)
                return result
            except AuthException as e1:
                typer.echo(__('Auth exception, change token'))
                app_logger.ex(f"Auth exception, change token - {e1}")
                raise typer.Exit(1)
            except BusinessLogicException as e2:
                typer.echo(__('We have error on business logic, please connect with developer'))
                app_logger.info(f"We have error on business logic, please connect with developer - {e2}")
                raise typer.Exit(1)
            except ConfigException as e3:
                typer.echo(__(
                    'We have error on working with config - %error_message%',
                    {
                        'error_message': e3.get_message()
                    }
                ))
                app_logger.info(f"We have error on working with config - {e3}")
                raise typer.Exit(1)
            except FileSystemException as e4:
                typer.echo(__(
                    "We have error on working with file system - %error_message%",
                    {
                        'error_message': e4.get_message()
                    }
                ))
                app_logger.info(f'We have error on working with file system - {e4}')
                raise typer.Exit(1)
            except HttpClientException as e5:
                typer.echo(__(
                    f"Error connect to the %stage_url%. %error_message%",
                    {
                        'stage_url': THESTAGE_API_URL,
                        'error_message': e5.get_message()
                    }
                ))
                app_logger.info(f"Error connect to the {THESTAGE_API_URL} - {e5}")
                raise typer.Exit(1)
            except GitAccessException as e6:
                typer.echo(e6.get_message())
                typer.echo(e6.get_dop_message())
                typer.echo(__(
                    "Please open this repo url %git_url% and 'Accept invitation', or check your access to repository",
                    {
                        'git_url': e6.get_url(),
                    }
                ))
                raise typer.Exit(1)
            except GitCommandError as e7:
                typer.echo(__(
                    "Error git command - %git_status% - %git_error%",
                    {
                        'git_status': e7.status,
                        'git_error': e7.stderr,
                    }
                ))
                app_logger.info(f"Error git command - {e7}")
                raise typer.Exit(1)
            except RemoteServerException as e8:
                typer.echo(__(
                    'Error connect to server or docker container %ip_address% by %username%',
                    {
                        'ip_address': e8.ip_address,
                        'username': e8.username,
                    }))
                app_logger.info(f'Error connect to server or docker container {e8.ip_address} by {e8.username} - {e8}')
                raise typer.Exit(1)
            except Exception as e100:
                if isinstance(e100, Exit):
                    raise e100
                else:
                    typer.echo(__('Undefined utility error'))
                    typer.echo(e100)
                    app_logger.info(f'Undefined utility error - {e100}')
                    raise typer.Exit(1)

        return wrapper
    return wrap



from typing import Optional, Dict, Tuple, List

from thestage.services.clients.thestage_api.dtos.enums.rented_status import RentedStatusEnumDto
from thestage.i18n.translation import __
from thestage.services.storage.mapper.storage_mapper import StorageMapper
from thestage.services.storage.storage_service import StorageService
from thestage.controllers.utils_controller import get_current_directory, base_global_check_validation

import typer


app = typer.Typer(no_args_is_help=True, help=__("Help working with rented storages"))


@app.command(name="list")
def item_list(
        row: int = typer.Option(
            5,
            '--row',
            '-r',
            help=__("Count row in table"),
            is_eager=False,
        ),
        page: int = typer.Option(
            1,
            '--page',
            '-p',
            help=__("Page number"),
            is_eager=False,
        ),
        statuses: List[RentedStatusEnumDto] = typer.Option(
            ["RENTED"],
            '--status',
            '-s',
            help=__("Status item (ALL - show all instances)"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        List rented storages
    """
    path = get_current_directory()

    config, facade = base_global_check_validation(path=path, no_dialog=no_dialog)

    headers = ['#', 'SLUG', 'TITLE', 'PROVIDER ID', 'STATUS', 'IS ACTIVE', 'CREATED AT', 'UPDATED AT']

    storage_service: StorageService = facade.get_storage_service()

    typer.echo(__(
        "Start show rented storages list with statuses: %statuses% (You can show all statuses set up status=ALL)",
        placeholders={
            'statuses': ', '.join([item.value for item in statuses])
        }))

    if RentedStatusEnumDto.find_special_status(statuses=statuses):
        statuses = []

    storage_service.print(
        func_get_data=storage_service.get_list,
        func_special_params={
            'statuses': statuses,
        },
        mapper=StorageMapper(),
        config=config,
        headers=headers,
        row=row,
        page=page,
        no_dialog=no_dialog,
    )

    typer.echo(__("List storages done"))
    raise typer.Exit(0)



from enum import Enum


class LocationRegionEnumDto(str, Enum):
    NORTH_AMERICA: str = 'NORTH_AMERICA'
    EUROPE: str = 'EUROPE'
    ASIA: str = 'ASIA'
    AFRICA: str = 'AFRICA'
    OCEANIA: str = 'OCEANIA'
    SOUTH_AMERICA: str = 'SOUTH_AMERICA'
    UNKNOWN: str = 'UNKNOWN'



import typer

from . import __app_name__

from thestage.controllers import base_controller, container_controller, instance_controller, storage_controller, config_controller

base_controller.app.add_typer(container_controller.app, name="container")
base_controller.app.add_typer(instance_controller.app, name="instance")
base_controller.app.add_typer(config_controller.app, name="config")
#base_controller.app.add_typer(storage_controller.app, name="storage")


def main():
    import thestage.config
    base_controller.app(prog_name=__app_name__)



from typing import Tuple, List, Optional, Any, Dict

import typer
from thestage_core.entities.config_entity import ConfigEntity

from thestage.helpers.logger.app_logger import app_logger
from thestage.i18n.translation import __
from thestage.services.clients.thestage_api.dtos.enums.selfhosted_status import SelfHostedBusinessStatusEnumDto, \
    SelfHostedFrontendStatusEnumDto
from thestage.services.clients.thestage_api.dtos.enums.rented_status import RentedBusinessStatusEnumDto, \
    RentedFrontendStatusEnumDto
from thestage.services.abstract_service import AbstractService
from thestage.helpers.error_handler import error_handler
from thestage.services.clients.thestage_api.api_client import TheStageApiClient
from thestage.services.clients.thestage_api.dtos.instance_rented_response import InstanceRentedDto, \
    InstanceRentedBusinessStatusMapperResponse
from thestage.services.clients.thestage_api.dtos.selfhosted_instance_response import SelfHostedInstanceDto, \
    SelfHostedRentedRentedBusinessStatusMapperResponse
from thestage.services.config_provider.config_provider import ConfigProvider
from thestage.services.remote_server_service import RemoteServerService


class InstanceService(AbstractService):

    __thestage_api_client: TheStageApiClient = None

    def __init__(
            self,
            thestage_api_client: TheStageApiClient,
            config_provider: ConfigProvider,
            remote_server_service: RemoteServerService,
    ):
        super(InstanceService, self).__init__(
            config_provider=config_provider
        )
        self.__thestage_api_client = thestage_api_client
        self.__remote_server_service = remote_server_service

    def get_rented_item(
            self,
            config: ConfigEntity,
            instance_slug: str,
    ) -> Optional[InstanceRentedDto]:
        return self.__thestage_api_client.get_rented_item(
            token=config.main.auth_token,
            instance_slug=instance_slug,
        )

    def get_self_hosted_item(
            self,
            config: ConfigEntity,
            instance_slug: str,
    ) -> Optional[SelfHostedInstanceDto]:
        return self.__thestage_api_client.get_selfhosted_item(
            token=config.main.auth_token,
            instance_slug=instance_slug,
        )

    @error_handler()
    def check_instance_status_to_connect(
            self,
            instance: InstanceRentedDto,
    ) -> InstanceRentedDto:
        if instance:
            if instance.frontend_status.status_key in [
                RentedBusinessStatusEnumDto.IN_QUEUE.name,
                RentedBusinessStatusEnumDto.CREATING.name,
                RentedBusinessStatusEnumDto.REBOOTING.name,
                RentedBusinessStatusEnumDto.STARTING.name,
            ]:
                typer.echo(__('Instance start renting or rebooting, please connect late'))
                raise typer.Exit(1)
            elif instance.frontend_status.status_key in [
                RentedBusinessStatusEnumDto.TERMINATING.name,
                RentedBusinessStatusEnumDto.RENTAL_ERROR.name,
            ]:
                typer.echo(__('Instance is failed, please start him'))
                raise typer.Exit(1)
            elif instance.frontend_status.status_key in [
                RentedBusinessStatusEnumDto.STOPPED.name,
                RentedBusinessStatusEnumDto.STOPPING.name,
                RentedBusinessStatusEnumDto.DELETED.name,
            ]:
                typer.echo(__('Instance is stopped or deleted, please create new'))
                raise typer.Exit(1)
            elif instance.frontend_status.status_key in [
                RentedBusinessStatusEnumDto.UNKNOWN.name,
                RentedBusinessStatusEnumDto.ALL.name,
            ]:
                typer.echo(__('Instance status unknown'))
                raise typer.Exit(1)

        return instance

    @error_handler()
    def check_selfhosted_status_to_connect(
            self,
            instance: SelfHostedInstanceDto,
    ) -> SelfHostedInstanceDto:
        if instance:
            if instance.frontend_status.status_key in [
                SelfHostedBusinessStatusEnumDto.AWAITING_CONFIGURATION.name,
            ]:
                typer.echo(__('Instance awaiting to configuration'))
                raise typer.Exit(1)
            elif instance.frontend_status.status_key in [
                SelfHostedBusinessStatusEnumDto.TERMINATED.name,
                SelfHostedBusinessStatusEnumDto.DELETED.name,
            ]:
                typer.echo(__('Instance is failed or deleted, please start him'))
                raise typer.Exit(1)
            elif instance.frontend_status.status_key in [
                SelfHostedBusinessStatusEnumDto.UNKNOWN.name,
                SelfHostedBusinessStatusEnumDto.ALL.name,
            ]:
                typer.echo(__('Instance status unknown'))
                raise typer.Exit(1)

        return instance

    @error_handler()
    def connect_to_instance(
            self,
            ip_address: str,
            username: str,
    ):
        self.__remote_server_service.connect_to_instance(
            ip_address=ip_address,
            username=username,
        )

    @error_handler()
    def get_rented_list(
            self,
            config: ConfigEntity,
            statuses: List[str],
            row: int = 5,
            page: int = 1,
    ) -> Tuple[List[Any], int]:
        data, total_pages = self.__thestage_api_client.get_rented_instance_list(
            token=config.main.auth_token,
            statuses=statuses,
            page=page,
            limit=row,
        )

        return data, total_pages

    @error_handler()
    def get_self_hosted_list(
            self,
            config: ConfigEntity,
            statuses: List[str],
            row: int = 5,
            page: int = 1,
    ) -> Tuple[List[Any], int]:
        data, total_pages = self.__thestage_api_client.get_selfhosted_instance_list(
            token=config.main.auth_token,
            statuses=statuses,
            page=page,
            limit=row,
        )
        return data, total_pages

    def map_rented_statuses(self, config: ConfigEntity, frontend: List[RentedFrontendStatusEnumDto]) -> Optional[List[str]]:
        statuses_mapper: Optional[Dict[str, str]] = self.__thestage_api_client.get_rented_business_status_map(token=config.main.auth_token,)
        return self.map_frontend_statuses(statuses_mapper=statuses_mapper, frontend=[item.value for item in frontend])

    def map_rented_statuses_old(self, config: ConfigEntity, frontend: List[RentedFrontendStatusEnumDto]) -> Optional[List[str]]:



        real_statuses = []

        statuses_mapper: Optional[InstanceRentedBusinessStatusMapperResponse] = self.__thestage_api_client.get_rented_business_status_map(token=config.main.auth_token,)

        if not statuses_mapper:
            return None

        mapper: Dict[str, List] = {}
        for key, value in statuses_mapper.instance_rented_business_status_map.items():

            map_key = value if ' ' not in value else value.replace(' ', '_')
            if value not in mapper:
                mapper[map_key] = [key]
            else:
                mapper[map_key].append(key)

        for item in frontend:
            if item == RentedFrontendStatusEnumDto.all:
                return []
            elif item == RentedFrontendStatusEnumDto.unknown:
                continue
            else:
                if item.name in mapper:
                    real_statuses.extend(mapper[item.name])
                else:
                    app_logger.error(f'Not found status on mapper rented statuses')

        return real_statuses

    def map_selfhosted_statuses(self, config: ConfigEntity, frontend: List[SelfHostedFrontendStatusEnumDto]) -> Optional[List[str]]:
        statuses_mapper: Optional[Dict[str, str]] = self.__thestage_api_client.get_selfhosted_business_status_map(token=config.main.auth_token,)
        return self.map_frontend_statuses(statuses_mapper=statuses_mapper, frontend=[item.value for item in frontend])

    def map_selfhosted_statuses_old(self, config: ConfigEntity, frontend: List[SelfHostedFrontendStatusEnumDto]) -> Optional[List[str]]:

        real_statuses = []

        statuses_mapper: Optional[SelfHostedRentedRentedBusinessStatusMapperResponse] = self.__thestage_api_client.get_selfhosted_business_status_map(token=config.main.auth_token,)

        if not statuses_mapper:
            return None

        mapper: Dict[str, List] = {}
        for key, value in statuses_mapper.selfhosted_instance_business_status_map.items():
            map_key = value if ' ' not in value else value.replace(' ', '_')
            if value not in mapper:
                mapper[map_key] = [key]
            else:
                mapper[map_key].append(key)

        for item in frontend:
            if item == SelfHostedFrontendStatusEnumDto.all:
                return []
            elif item == SelfHostedFrontendStatusEnumDto.unknown:
                continue
            else:
                if item.name in mapper:
                    real_statuses.extend(mapper[item.name])
                else:
                    app_logger.error(f'Not found status on mapper selfhosted statuses')

        return real_statuses



from enum import Enum
from typing import List


class ContainerBussinessStatusEnumDto(str, Enum):
    FAILED: str = 'FAILED'
    BUSY: str = 'BUSY'
    DEAD: str = 'DEAD'
    CREATING: str = 'CREATING'
    CREATING_FAILED: str = 'CREATING_FAILED'
    STARTING: str = 'STARTING'
    RUNNING: str = 'RUNNING'
    STOPPING: str = 'STOPPING'
    STOPPED: str = 'STOPPED'
    RESTARTING: str = 'RESTARTING'
    DELETING: str = 'DELETING'
    DELETED: str = 'DELETED'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['ContainerBussinessStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == ContainerBussinessStatusEnumDto.ALL else False, statuses))
        return True if q else False


class ContainerFrontendStatusEnumDto(str, Enum):
    terminated: str = 'terminated'
    unresponsive: str = 'unresponsive'
    creation_failed: str = 'creation_failed'
    exited: str = 'exited'
    starting: str = 'starting'
    running: str = 'running'
    terminating: str = 'terminating'
    stopped: str = 'stopped'
    stopping: str = 'stopping'
    unknown: str = 'unknown'
    all: str = 'all'

    @staticmethod
    def find_special_status(statuses: List['ContainerFrontendStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == ContainerFrontendStatusEnumDto.all else False, statuses))
        return True if q else False



import os
from typing import List, Tuple, Optional, Dict

import typer
from thestage_core.entities.config_entity import ConfigEntity
from thestage_core.services.filesystem_service import FileSystemServiceCore

from thestage.helpers.logger.app_logger import app_logger
from thestage.services.clients.thestage_api.dtos.container_param_request import DockerContainerActionRequestDto
from thestage.services.clients.thestage_api.dtos.enums.container_pending_action import ContainerPendingActionEnumDto
from thestage.services.clients.thestage_api.dtos.enums.container_status import ContainerBussinessStatusEnumDto, \
    ContainerFrontendStatusEnumDto
from thestage.entities.enums.shell_type import ShellType
from thestage.services.clients.thestage_api.dtos.sftp_path_helper import SftpPathHelper
from thestage.services.remote_server_service import RemoteServerService
from thestage.i18n.translation import __
from thestage.services.abstract_service import AbstractService
from thestage.services.clients.thestage_api.dtos.container_response import DockerContainerDto
from thestage.helpers.error_handler import error_handler
from thestage.services.clients.thestage_api.api_client import TheStageApiClient
from thestage.services.config_provider.config_provider import ConfigProvider


class ContainerService(AbstractService):

    __thestage_api_client: TheStageApiClient = None

    def __init__(
            self,
            thestage_api_client: TheStageApiClient,
            config_provider: ConfigProvider,
            remote_server_service: RemoteServerService,
            file_system_service: FileSystemServiceCore,
    ):
        super(ContainerService, self).__init__(
            config_provider=config_provider
        )
        self.__thestage_api_client = thestage_api_client
        self.__remote_server_service = remote_server_service
        self.__file_system_service = file_system_service

    @error_handler()
    def get_list(
            self,
            config: ConfigEntity,
            statuses: List[str],
            row: int = 5,
            page: int = 1,
    ) -> Tuple[List[DockerContainerDto], int]:

        data, total = self.__thestage_api_client.get_container_list(
            token=config.main.auth_token,
            statuses=statuses,
            page=page,
            limit=row,
        )

        return data, total

    @error_handler()
    def get_item(
            self,
            config: ConfigEntity,
            container_id: Optional[int] = None,
            container_slug: Optional[str] = None,
    ) -> Optional[DockerContainerDto]:
        return self.__thestage_api_client.get_container_item(
            token=config.main.auth_token,
            container_id=container_id,
            container_slug=container_slug,
        )

    @staticmethod
    def get_server_auth(
            container: DockerContainerDto,
            no_dialog: bool = False,
            username_param: Optional[str] = None,
    ) -> Tuple[str, str]:
        if container.instance_rented:
            username = container.instance_rented.host_username
            ip_address = container.instance_rented.ip_address
        elif container.selfhosted_instance:
            username = container.selfhosted_instance.host_username or 'root'
            ip_address = container.selfhosted_instance.ip_address
        else:
            typer.echo(__("Not found rented or selfhosted instance for connect"))
            raise typer.Exit(1)

        if not username_param:
            if not username:
                if no_dialog:
                    typer.echo(__('Please set up username as parameter on no-dialog mode'))
                    raise typer.Exit(1)

                username = typer.prompt(
                    default='ubuntu',
                    text=__('Please set up username to connect instance'),
                    show_choices=False,
                    type=str,
                    show_default=True,
                )
        else:
            username = username_param

        return username, ip_address

    @error_handler()
    def connect_container(
            self,
            config: ConfigEntity,
            container: DockerContainerDto,
            no_dialog: bool = False,
            username_param: Optional[str] = None,
    ):

        username, ip_address = self.get_server_auth(
            container=container,
            username_param=username_param,
            no_dialog=no_dialog,
        )

        if not container.system_name:
            typer.echo(__("Container name not present, can not connect to container"))
            raise typer.Exit(1)

        shell: Optional[ShellType] = self.__remote_server_service.get_shell_from_container(
            ip_address=ip_address,
            username=username,
            docker_name=container.system_name,
        )

        if not shell:
            typer.echo(__("We can not get shell on container (bash, shell), please use standard shell"))
            raise typer.Exit(1)

        self.__remote_server_service.connect_to_container(
            ip_address=ip_address,
            username=username,
            docker_name=container.system_name,
            shell=shell,
        )

    @error_handler()
    def check_container_status_for_start(
            self,
            container: DockerContainerDto,
    ) -> DockerContainerDto:
        if container:
            if container.status == ContainerBussinessStatusEnumDto.RUNNING:
                typer.echo(__('Container is running, can not start working server'))
                raise typer.Exit(1)
            elif container.status == ContainerBussinessStatusEnumDto.CREATING:
                typer.echo(__('Container creating, please check status later'))
                raise typer.Exit(1)
            elif container.status in [
                ContainerBussinessStatusEnumDto.STARTING,
                ContainerBussinessStatusEnumDto.RESTARTING,
            ]:
                typer.echo(__('The container starts now, you can be up and running in minutes'))
                raise typer.Exit(1)
            elif container.status in [
                ContainerBussinessStatusEnumDto.DELETING,
                ContainerBussinessStatusEnumDto.DELETED,
            ]:
                typer.echo(__('The container was deleted, create new'))
                raise typer.Exit(1)

        return container

    @error_handler()
    def check_container_status_for_stop(
            self,
            container: DockerContainerDto,
    ) -> DockerContainerDto:
        if container:
            if container.status in [
                ContainerBussinessStatusEnumDto.CREATING,
                ContainerBussinessStatusEnumDto.STARTING,
                ContainerBussinessStatusEnumDto.RESTARTING,
            ]:
                typer.echo(__('Container in rebooting or creating, please stop later'))
                raise typer.Exit(1)
            elif container.status in [
                ContainerBussinessStatusEnumDto.FAILED,
                ContainerBussinessStatusEnumDto.DEAD,
                ContainerBussinessStatusEnumDto.CREATING_FAILED,
                ContainerBussinessStatusEnumDto.STOPPING,
                ContainerBussinessStatusEnumDto.STOPPED,
            ]:
                typer.echo(__('Container was stopped, by error or manually'))
                raise typer.Exit(1)
            elif container.status in [
                ContainerBussinessStatusEnumDto.DELETING,
                ContainerBussinessStatusEnumDto.DELETED,
            ]:
                typer.echo(__('The container was deleted, create new'))
                raise typer.Exit(1)

        return container

    @error_handler()
    def check_container_status_for_work(
            self,
            container: DockerContainerDto,
    ) -> DockerContainerDto:
        if container:
            if container.frontend_status.status_key in [
                ContainerBussinessStatusEnumDto.CREATING.value,
                ContainerBussinessStatusEnumDto.STARTING.value,
                ContainerBussinessStatusEnumDto.RESTARTING.value,
            ]:
                typer.echo(__('Container is rebooted, please connect late'))
                raise typer.Exit(1)
            elif container.frontend_status.status_key in [
                ContainerBussinessStatusEnumDto.FAILED.value,
                ContainerBussinessStatusEnumDto.DEAD.value,
                ContainerBussinessStatusEnumDto.CREATING_FAILED.value,
                ContainerBussinessStatusEnumDto.STOPPING.value,
                ContainerBussinessStatusEnumDto.STOPPED.value,
            ]:
                typer.echo(__('Container is failed, please start him'))
                raise typer.Exit(1)
            elif container.frontend_status.status_key in [
                ContainerBussinessStatusEnumDto.DELETING.value,
                ContainerBussinessStatusEnumDto.DELETED.value,
            ]:
                typer.echo(__('Container was deleted, create new'))
                raise typer.Exit(1)

        return container

    @error_handler()
    def put_file_to_container(
            self,
            container: DockerContainerDto,
            src_path: str,
            destination_path: Optional[str] = None,
            is_folder: bool = False,
            no_dialog: bool = False,
            username_param: Optional[str] = None,
    ):

        username, ip_address = self.get_server_auth(
            container=container,
            username_param=username_param,
            no_dialog=no_dialog,
        )

        if not self.__file_system_service.check_if_path_exist(file=src_path):
            typer.echo(__("Not found path with file"))
            raise typer.Exit(1)

        if (not is_folder) and self.__file_system_service.is_folder(folder=src_path, auto_create=False, with_exception=False):
            typer.echo(__("Path is folder, please set up recursive flag"))
            raise typer.Exit(1)

        if container.mappings and container.mappings.directory_mappings:
            path_helper: SftpPathHelper = self.__remote_server_service.check_path_mapping(
                directory_mapping=container.mappings.directory_mappings,
                path=destination_path,
                is_upload=True,
                is_folder=is_folder,
            )
        else:
            typer.echo(__("Error, can not find mapping folders"))
            raise typer.Exit(1)

        if not path_helper.tmp_instance_path:
            typer.echo(__("Error, can not find mapping for /public folder"))
            raise typer.Exit(1)

        self.__remote_server_service.upload_data_to_container(
            ip_address=ip_address,
            username=username,
            docker_name=container.system_name,
            src_path=src_path,
            dest_path=destination_path,
            path_helper=path_helper,
            #real_path=real_path,
            #client_path=destination_path,
            #temporary_folder_path=temporary_folder_path,
            is_folder=is_folder,
        )

    @error_handler()
    def get_file_from_container(
            self,
            container: DockerContainerDto,
            src_path: str,
            destination_path: Optional[str] = None,
            is_folder: bool = False,
            no_dialog: bool = False,
            username_param: Optional[str] = None,
    ):
        username, ip_address = self.get_server_auth(
            container=container,
            username_param=username_param,
            no_dialog=no_dialog,
        )

        #if not self.__file_system_service.check_if_path_exist(file=destination_path):
        #    typer.echo(__("Not found path with file"))
        #    raise typer.Exit(1)

        if container.mappings and container.mappings.directory_mappings:
            path_helper: SftpPathHelper = self.__remote_server_service.check_path_mapping(
                directory_mapping=container.mappings.directory_mappings,
                path=src_path,
                is_upload=False,
                is_folder=is_folder,
            )
        else:
            typer.echo(__("Error, can not find mapping folders"))
            raise typer.Exit(1)

        if not path_helper.tmp_container_path:
            typer.echo(__("Error, can not find path to copy"))
            raise typer.Exit(1)

        self.__remote_server_service.download_data_to_container(
            ip_address=ip_address,
            username=username,
            docker_name=container.system_name,
            src_path=src_path,
            dest_path=destination_path,
            path_helper=path_helper,
            is_folder=is_folder,
        )

    @error_handler()
    def change_container_status(
            self,
            config: ConfigEntity,
            container: DockerContainerDto,
            action: ContainerPendingActionEnumDto,
    ) -> bool:
        request_params = DockerContainerActionRequestDto(
            dockerContainerId=container.id,
            action=action,
        )
        result = self.__thestage_api_client.container_action(
            token=config.main.auth_token,
            request_param=request_params,
        )
        if not result:
            app_logger.error(f'Container dont change status - {result}')

        return result

    def map_container_statuses(self, config: ConfigEntity, frontend: List[ContainerFrontendStatusEnumDto]) -> Optional[List[str]]:
        statuses_mapper: Optional[Dict[str, str]] = self.__thestage_api_client.get_container_business_status_map(token=config.main.auth_token,)
        return self.map_frontend_statuses(statuses_mapper=statuses_mapper, frontend=[item.value for item in frontend])



import os
import stat
import uuid
from pathlib import Path
from time import sleep
from typing import Optional, List, Dict, Tuple

import paramiko
import typer
from paramiko.client import SSHClient
from paramiko.config import SSHConfig
from paramiko.hostkeys import HostKeys
from paramiko.sftp_attr import SFTPAttributes
from paramiko.sftp_client import SFTPClient
from thestage_core.entities.file_item import FileItemEntity
from thestage_core.services.filesystem_service import FileSystemServiceCore

from thestage.exceptions.remote_server_exception import RemoteServerException
from thestage.helpers.logger.app_logger import app_logger
from thestage.entities.enums.shell_type import ShellType
from thestage.i18n.translation import __
from thestage.services.clients.thestage_api.dtos.sftp_path_helper import SftpPathHelper

old_value: int = 0


class RemoteServerService(object):

    def __init__(
            self,
            file_system_service: FileSystemServiceCore,
    ):
        self.__file_system_service = file_system_service

    def __get_client(
            self,
            ip_address: str,
            username: str,
    ) -> Optional[SSHClient]:
        config_by_ip = None
        ssh_path = self.__file_system_service.get_ssh_path()
        ssh_config_path = ssh_path.joinpath('config')
        #ssh_config_path = Path('~/.ssh/config')
        config_path = ssh_config_path.expanduser()
        if config_path.exists():
            config = SSHConfig.from_path(config_path)
            config_by_ip = config.lookup(ip_address)
        client = SSHClient()
        try:
            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            if config_by_ip:
                key_file_name = config_by_ip['identityfile'][0] if config_by_ip and 'identityfile' in config_by_ip and len(config_by_ip['identityfile']) > 0 else None
                client.connect(
                    hostname=ip_address,
                    username=username,
                    timeout=60,
                    allow_agent=False if key_file_name else True,
                    look_for_keys=False if key_file_name else True,
                    key_filename=key_file_name.replace('.pub', '') if key_file_name else None,
                )
            else:
                client.connect(
                    hostname=ip_address,
                    username=username,
                    timeout=60,
                )
            return client

        except Exception as ex:
            if client:
                client.close()
                typer.echo(f"Error connect to {ip_address} by {username} ({ex})")
            app_logger.error(f"Error connect to {ip_address} by {username} ({ex})")
            raise RemoteServerException(
                message=__("Error connect to remote server"),
                ip_address=ip_address,
                username=username,
            )

    @staticmethod
    def is_shell_present(
            lines: List[str],
    ) -> Optional[ShellType]:
        bash_present, sh_present = False, False
        for line in lines:
            if 'bin/bash' in line:
                bash_present = True
                break
            elif 'bin/sh' in line:
                sh_present = True

        if bash_present:
            return ShellType.BASH
        if sh_present:
            return ShellType.SH
        else:
            return None

    def get_shell_from_container(
            self,
            ip_address: str,
            username: str,
            docker_name: str,
    ) -> Optional[ShellType]:
        client: Optional[SSHClient] = self.__get_client(ip_address=ip_address, username=username)
        stdin, stdout, stderr = client.exec_command(f'docker exec -it {docker_name} cat /etc/shells', get_pty=True)
        shell = self.is_shell_present(lines=stdout.readlines())
        client.close()

        return shell

    def check_if_host_in_list_known(self, ip_address: str) -> bool:
        try:
            #os.system(f"ssh-keygen -F 34.198.189.175")
            ssh_path = self.__file_system_service.get_ssh_path()
            known_host_path = ssh_path.joinpath('known_hosts')
            if known_host_path.exists():
                host_keys = HostKeys(filename=str(known_host_path.absolute()))
                result = host_keys.lookup(ip_address)
                if result is not None:
                    return True
                else:
                    return False
        except Exception as ex:
            raise FileNotFoundError(f"Error read ssh known host file: {ex}")
        return False

    def connect_to_instance(
            self,
            ip_address: str,
            username: str,
    ):
        try:
            os.system(f"ssh {username}@{ip_address}")
        except Exception as ex:
            app_logger.error(f"Error connect to {ip_address} by {username} ({ex})")
            raise RemoteServerException(
                message=__("Error connect to remote server"),
                ip_address=ip_address,
                username=username,
            )

    def connect_to_container(
            self,
            ip_address: str,
            username: str,
            docker_name: str,
            shell: ShellType
    ):
        try:
            os.system(f"ssh -tt {username}@{ip_address} 'docker exec -it {docker_name} {shell.value}'")
        except Exception as ex:
            app_logger.exception(f"Error connect to {ip_address} by {username} ({ex})")
            raise RemoteServerException(
                message=__("Error connect to remote server"),
                ip_address=ip_address,
                username=username,
            )

    @staticmethod
    def find_path_mapping(
            directory_mapping: Dict[str, str],
            destination_path: Optional[str] = None,
    ) -> Optional[str]:

        template = destination_path if destination_path else '/public'

        if not directory_mapping:
            typer.echo(__("Can not find mapping folders"))
            raise typer.Exit(1)

        for key, value in directory_mapping.items():
            if value == template:
                return key
        return None

    @staticmethod
    def check_path_mapping(
            directory_mapping: Dict[str, str],
            path: Optional[str],
            is_upload: bool = True,
            is_folder: bool = False,
    ) -> Optional[SftpPathHelper]:
        if not directory_mapping:
            typer.echo(__("Can not find mapping folders"))
            raise typer.Exit(1)

        public_src, public_desc = None, None

        for key, value in directory_mapping.items():
            #template = value if is_upload else key
            template = value

            if value == '/public':
                #public_src = key if is_upload else value
                #public_desc = value if is_upload else key
                public_src = value
                public_desc = key

            if template in path:
                path_list = path.split('/')
                return SftpPathHelper(
                    tmp_container_path='/'.join(path_list[0:-1]),
                    tmp_instance_path='/'.join(path.replace(template, key).split('/')[0:-1]),
                    item_name=path_list[-1],
                )

        if public_src == None:
            typer.echo(__("Can't found public folder"))
            raise typer.Exit(1)

        tmp_folder = f"{uuid.uuid4()}"
        path_list = path.split('/')
        last = path_list[-1]

        return SftpPathHelper(
            tmp_container_path=f"{public_src}/{tmp_folder}",
            tmp_instance_path=f"{public_desc}/{tmp_folder}",
            tmp_folder_path=f"{public_src}/{tmp_folder}",
            item_name=last,
        )

    def __upload_one_file(
            self,
            sftp: SFTPClient,
            src_path: str,
            dest_path: str,
            file_name: str,
            file_size: [int] = 100
    ) -> bool:
        has_error = False
        try:
            with typer.progressbar(length=file_size, label=__("Copy %file_name%", {'file_name': file_name})) as progress:
                def __show_result_copy(size: int, full_size: int):
                    global old_value
                    progress.update(size - (old_value or 0))
                    old_value = size
                sftp.put(localpath=src_path, remotepath=f"{dest_path}", callback=__show_result_copy)
            typer.echo(__('Complete copy %file_name%', {'file_name': file_name}))
        except FileNotFoundError as err:
            app_logger.exception(f"Error put file {file_name} to container (file ot found): {err}")
            typer.echo(__("Error put file, file not found on server"))
            has_error = True
        except Exception as err2:
            typer.echo(err2)
            app_logger.exception(f"Error put file {file_name} to container: {err2}")
            typer.echo(__("Error put file, undefined server error"))
            has_error = True

        return has_error

    def __make_dirs_by_sftp(
            self,
            sftp: SFTPClient,
            path: str,
    ):
        full_path = ''
        for item in path.split('/'):
            if item == '':
                continue
            try:
                full_path += f'/{item}'
                sftp.chdir(full_path)  # Test if remote_path exists
            except IOError:
                sftp.mkdir(full_path)  # Create remote_path
                sftp.chdir(full_path)

    def __upload_list_files(
            self,
            sftp: SFTPClient,
            src_item: FileItemEntity,
            dest_path: str,
    ):
        if src_item.is_file:
            # need create folder if use dynamic folder
            paths = dest_path.split('/')
            new_name_file = paths[-1]
            real_dest_path = '/'.join(paths[0:-1])
            self.__make_dirs_by_sftp(sftp=sftp, path=real_dest_path)

            self.__upload_one_file(
                sftp=sftp,
                src_path=src_item.path,
                dest_path=dest_path,
                file_name=new_name_file,
                file_size=src_item.file_size,
            )
        elif src_item.is_folder:
            self.__make_dirs_by_sftp(sftp=sftp, path=dest_path)
            #server_dir = f"{dest_path}/{src_item.name}"
            #self.__make_dirs_by_sftp(sftp=sftp, path=server_dir)

            for item in src_item.children:
                self.__upload_list_files(
                    sftp=sftp,
                    src_item=item,
                    dest_path=f"{dest_path}/{item.name}",
                )

    def __download_one_file(
            self,
            sftp: SFTPClient,
            src_path: str,
            dest_path: str,
            file_name: str,
            file_size: [int] = 100
    ) -> bool:
        has_error = False
        try:
            with typer.progressbar(length=file_size, label=__("Copy %file_name%", {'file_name': file_name})) as progress:
                def __show_result_copy(size: int, full_size: int):
                    global old_value
                    progress.update(size - (old_value or 0))
                    old_value = size
                sftp.get(remotepath=src_path, localpath=f"{dest_path}", callback=__show_result_copy)
            typer.echo(__('Complete copy %file_name%', {'file_name': file_name}))
        except FileNotFoundError as err:
            app_logger.exception(f"Error get file {file_name} from container (file ot found): {err}")
            typer.echo(__("Error get file, file not found on server"))
            has_error = True
        except Exception as err2:
            typer.echo(err2)
            app_logger.exception(f"Error get file {file_name} from container: {err2}")
            typer.echo(__("Error get file, undefined server error"))
            has_error = True

        return has_error

    def __download_list_files(
            self,
            sftp: SFTPClient,
            src_item: FileItemEntity,
            dest_path: str,
    ):
        if src_item.is_file:
            self.__file_system_service.get_path('/'.join(dest_path.split('/')[0:-1]), auto_create=True)
            self.__download_one_file(
                sftp=sftp,
                src_path=src_item.path,
                dest_path=dest_path,
                file_name=src_item.name,
                file_size=src_item.file_size,
            )
        elif src_item.is_folder:
            server_dir = self.__file_system_service.get_path(str(Path(dest_path).joinpath(src_item.name)), auto_create=True)
            for item in src_item.children:
                self.__download_list_files(
                    sftp=sftp,
                    src_item=item,
                    dest_path=str(server_dir.joinpath(item.name)) if item.is_file else str(server_dir),
                )

    @staticmethod
    def find_sftp_server_path(
            client: SSHClient,
    ) -> Optional[str]:
        stdin, stdout, stderr = client.exec_command(f'whereis sftp-server', get_pty=True)
        for line in stdout.readlines():
            pre_line = line.replace('sftp-server:', '')
            for command in pre_line.strip().split(' '):
                tmp = command.strip()
                if tmp:
                    if tmp.endswith('/sftp-server'):
                        return tmp
        return None

    def copy_data_on_container(
            self,
            client: SSHClient,
            docker_name: str,
            src_path: str,
            dest_path: str,
            is_recursive: bool = False,
    ):
        self.start_command_on_container(
            client=client,
            docker_name=docker_name,
            command='cp ' + ('-R' if is_recursive else '') + f' {src_path}' + f' {dest_path}',
        )
        # TODO: dont now how, need check for copy end!!!!
        sleep(3)

    @staticmethod
    def start_command_on_container(
            client: SSHClient,
            docker_name: str,
            command: str,
    ):
        stdin, stdout, stderr = client.exec_command(f'docker exec -it {docker_name} {command}', get_pty=True)
        for line in stdout.readlines():
            pass

    def upload_data_to_container(
            self,
            ip_address: str,
            username: str,
            docker_name: str,
            src_path: str,
            dest_path: str,
            path_helper: SftpPathHelper,
            is_folder: bool = False,
    ):
        has_error = False
        client: Optional[SSHClient] = self.__get_client(ip_address=ip_address, username=username)
        sftp_server_path = self.find_sftp_server_path(client=client)

        if not sftp_server_path:
            typer.echo(__('No sftp server is installed on your instance'))
            raise typer.Exit(1)

        chan = client.get_transport().open_session()
        #chan.exec_command("sudo su -c /usr/lib/openssh/sftp-server")
        chan.exec_command(f"sudo su -c {sftp_server_path}")
        sftp = paramiko.SFTPClient(chan)

        try:
            files: List[FileItemEntity] = self.__file_system_service.get_path_items(src_path)
            for item in files:
                self.__upload_list_files(
                    sftp=sftp,
                    src_item=item,
                    dest_path=path_helper.tmp_instance_path_with_name(),
                )

            if path_helper.tmp_folder_path:
                # copy to dest_path
                self.start_command_on_container(
                    client=client,
                    docker_name=docker_name,
                    command='cp ' + ('-R' if is_folder else '') + f' {path_helper.tmp_container_path_with_name()}' + f' {dest_path}',
                )

        except FileNotFoundError as err:
            app_logger.error(f"Error put file to container {ip_address}, {username} (file ot found): {err}")
            typer.echo(__("Error put file, file not found on server"))
            has_error = True
        finally:
            if path_helper.tmp_folder_path:
                self.start_command_on_container(
                    client=client,
                    docker_name=docker_name,
                    command='rm -R ' + f'{path_helper.tmp_folder_path}',
                )
            sftp.close()

        client.close()
        if has_error:
            typer.Exit(1)

    def __read_remote_path_items(self, sftp: SFTPClient, folder: str) -> List[FileItemEntity]:
        path_items = []
        root_stat = sftp.stat(folder)
        parent = FileItemEntity(
            name=folder.split('/')[-1],
            path=folder,
            is_file=stat.S_ISREG(root_stat.st_mode),
            is_folder=stat.S_ISDIR(root_stat.st_mode),
            file_size=root_stat.st_size,
        )
        path_items.append(parent)
        if parent.is_folder:
            sftp.chdir(folder)
            for item in sftp.listdir_attr():
                next_path = f'{folder}/{item.filename}'
                is_dir = stat.S_ISDIR(item.st_mode)
                is_file = stat.S_ISREG(item.st_mode)
                if is_file:
                    parent.children.append(FileItemEntity(
                        name=item.filename,
                        path=next_path,
                        is_file=is_file,
                        is_folder=is_dir,
                        file_size=item.st_size,
                    ))
                elif is_dir:
                    parent.children.extend(self.__read_remote_path_items(
                        sftp=sftp,
                        folder=next_path,
                    ))
        return path_items

    def download_data_to_container(
            self,
            ip_address: str,
            username: str,
            docker_name: str,
            src_path: str,
            dest_path: str,
            path_helper: SftpPathHelper,
            is_folder: bool = False,
    ):
        has_error = False
        client: Optional[SSHClient] = self.__get_client(ip_address=ip_address, username=username)

        sftp_server_path = self.find_sftp_server_path(client=client)

        if not sftp_server_path:
            typer.echo(__('No sftp server is installed on your instance'))
            raise typer.Exit(1)

        chan = client.get_transport().open_session()
        #chan.exec_command("sudo su -c /usr/lib/openssh/sftp-server")
        chan.exec_command(f"sudo su -c {sftp_server_path}")
        sftp = paramiko.SFTPClient(chan)

        try:
            if path_helper.tmp_folder_path:
                self.__make_dirs_by_sftp(sftp=sftp, path=path_helper.tmp_instance_path)
                self.copy_data_on_container(
                    client=client,
                    docker_name=docker_name,
                    is_recursive=is_folder,
                    src_path=src_path,
                    dest_path=path_helper.tmp_container_path if is_folder else path_helper.tmp_container_path_with_name(),
                )

            files: List[FileItemEntity] = self.__read_remote_path_items(
                sftp=sftp,
                folder=path_helper.tmp_instance_path_with_name(),
            )

            for item in files:
                self.__download_list_files(
                    sftp=sftp,
                    src_item=item,
                    dest_path=dest_path,
                )
        except FileNotFoundError as err:
            print(err)
            app_logger.error(f"Error put file to container {ip_address}, {username} (file ot found): {err}")
            typer.echo(__("Error put file, file not found on server"))
            has_error = True
        finally:
            if path_helper.tmp_folder_path:
                # remove temporary path
                self.start_command_on_container(
                    client=client,
                    docker_name=docker_name,
                    command='rm -R ' + f'{path_helper.tmp_folder_path}',
                )
            sftp.close()

        client.close()
        if has_error:
            typer.Exit(1)



from pydantic import BaseModel, Field


class SftpPathHelper(BaseModel):

    tmp_container_path: str = Field(None)
    tmp_instance_path: str = Field(None)
    tmp_folder_path: str = Field(None)
    item_name: str = Field(None)

    def tmp_container_path_with_name(self) -> str:
        return f"{self.tmp_container_path}/{self.item_name}"

    def tmp_instance_path_with_name(self) -> str:
        return f"{self.tmp_instance_path}/{self.item_name}"



from typing import Optional, List

from pydantic import Field, BaseModel, ConfigDict

from thestage.services.clients.thestage_api.dtos.enums.container_pending_action import ContainerPendingActionEnumDto
from thestage.services.clients.thestage_api.dtos.docker_container_mapping import DockerContainerMappingDto


class DockerContainerCreateRequestDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    fk_instance_rented: Optional[int] = Field(None, alias='fkInstanceRented')
    fk_selfhosted_instance: Optional[int] = Field(None, alias='fkSelfhostedInstance')
    fk_sketch: Optional[int] = Field(None, alias='fkSketch')

    title: Optional[str] = Field(None, alias='title')
    slug: Optional[str] = Field(None, alias='slug')
    docker_image: Optional[str] = Field(None, alias='dockerImage')
    assigned_cpus: Optional[int] = Field(None, alias='assignedCpus')
    assigned_gpu_ids: List[int] = Field(default_factory=list, alias='assignedGpuIds')
    mappings: Optional[DockerContainerMappingDto] = Field(None, alias='mappings')
    container_count: Optional[int] = Field(None, alias='containerCount')


class DockerContainerActionRequestDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    container_id: Optional[int] = Field(None, alias='dockerContainerId')
    action: ContainerPendingActionEnumDto = Field(ContainerPendingActionEnumDto.UNKNOWN, alias='action')



from typing import Optional

from pydantic import Field, BaseModel, ConfigDict

from thestage.services.clients.thestage_api.dtos.selfhosted_instance_response import SelfHostedInstanceDto
from thestage.services.clients.thestage_api.dtos.task_response import TaskDto
from thestage.services.clients.thestage_api.dtos.instance_rented_response import InstanceRentedDto
from thestage.services.clients.thestage_api.dtos.storage_rented_response import StorageRentedDto
from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse


class SketchDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    id: Optional[int] = Field(None, alias='id')
    client_id: Optional[int] = Field(None, alias='fkClient')
    fk_instance_rented_default: Optional[int] = Field(None, alias='fkInstanceRentedDefault')
    fk_selfhosted_instance_default: Optional[int] = Field(None, alias='fkSelfhostedInstanceDefault')
    fk_storage_rented_default: Optional[int] = Field(None, alias='fkStorageRentedDefault')
    ssh_key_deploy: Optional[int] = Field(None, alias='fkSshKeyDeploy')
    name: Optional[str] = Field(None, alias='name')
    slug: Optional[str] = Field(None, alias='slug')
    description: Optional[str] = Field(None, alias='description')
    github_username: Optional[str] = Field(None, alias='githubCollaboratorUsername')

    last_commit_hash: Optional[str] = Field(None, alias='lastCommitHash')
    last_commit_description: Optional[str] = Field(None, alias='lastCommitDescription')
    git_repository_url: Optional[str] = Field(None, alias='gitRepositoryUrl')
    git_repository_name: Optional[str] = Field(None, alias='gitRepositoryName')

    task_count: Optional[int] = Field(None, alias='taskCount')
    favourite_task_count: Optional[int] = Field(None, alias='favouriteTaskCount')

    last_task_run_date: Optional[str] = Field(None, alias='lastTaskRunDate')
    created_at: Optional[str] = Field(None, alias='createdAt')
    updated_at: Optional[str] = Field(None, alias='updatedAt')
    instance_rented_default: Optional[InstanceRentedDto] = Field(None, alias='instanceRentedDefault')
    selfhosted_instance_default: Optional[SelfHostedInstanceDto] = Field(None, alias='selfhostedInstanceDefault')
    storage_rented_default: Optional[StorageRentedDto] = Field(None, alias='storageRentedDefault')


class SketchViewResponse(TheStageBaseResponse):
    sketch: Optional[SketchDto] = Field(None, alias='sketch')


class SketchRunTaskResponse(TheStageBaseResponse):
    task: Optional[TaskDto] = Field(None, alias='task')



from typing import Optional

from pydantic import BaseModel, ConfigDict, Field


class SelfHostedInstanceEntity(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    id: Optional[int] = Field(None, alias='id')
    slug: Optional[str] = Field(None, alias='slug')
    title: Optional[str] = Field(None, alias='title')
    cpu_type: Optional[str] = Field(str, alias='cpu_type')
    cpu_cores: Optional[int] = Field(None, alias='cpu_cores')
    ram_size_gb: Optional[int] = Field(None, alias='ram_size_gb')
    ip_address: Optional[str] = Field(None, alias='ip_address')
    username: Optional[str] = Field(None, alias='username')
    status: Optional[str] = Field(None, alias='status')
    created_at: Optional[str] = Field(None, alias='created_at')
    updated_at: Optional[str] = Field(None, alias='updated_at')



from typing import Optional, List

from pydantic import Field, BaseModel, ConfigDict

from thestage.services.clients.thestage_api.dtos.selfhosted_instance_response import SelfHostedInstanceDto
from thestage.services.clients.thestage_api.dtos.enums.task_execution_status import TaskExecutionStatusEnumDto
from thestage.services.clients.thestage_api.dtos.instance_rented_response import InstanceRentedDto
from thestage.services.clients.thestage_api.dtos.pagination_data import PaginationData
from thestage.services.clients.thestage_api.dtos.storage_rented_response import StorageRentedDto
from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse


class TaskDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    id: Optional[int] = Field(None, alias='id')
    fk_instance_rented: Optional[int] = Field(None, alias='fkInstanceRented')
    fk_selfhosted_instance_default: Optional[int] = Field(None, alias='fkSelfhostedInstanceDefault')
    fk_storage_rented: Optional[int] = Field(None, alias='fkStorageRented')
    description: Optional[str] = Field(None, alias='description')
    title: Optional[str] = Field(None, alias='title')
    source_path: Optional[str] = Field(None, alias='sourcePath')
    source_target_path: Optional[str] = Field(None, alias='sourceTargetPath')
    data_path: Optional[str] = Field(None, alias='dataPath')
    result_destination_path: Optional[str] = Field(None, alias='resultDestinationPath')
    run_command: Optional[str] = Field(None, alias='runCommand')
    run_args: Optional[str] = Field(None, alias='runArgs')
    commit_hash: Optional[str] = Field(None, alias='commitHash')
    ordinal_number_for_sketch: Optional[int] = Field(None, alias='ordinalNumberForSketch')
    fk_user: Optional[int] = Field(None, alias='fkUser')
    status: TaskExecutionStatusEnumDto = Field(TaskExecutionStatusEnumDto.UNKNOWN, alias='status')
    created_at: Optional[str] = Field(None, alias='createdAt')
    updated_at: Optional[str] = Field(None, alias='updatedAt')
    started_at: Optional[str] = Field(None, alias='startedAt')
    finished_at: Optional[str] = Field(None, alias='finishedAt')
    client_id: Optional[int] = Field(None, alias='clientId')

    instance_rented: Optional[InstanceRentedDto] = Field(None, alias='instanceRented')
    selfhosted_instance: Optional[SelfHostedInstanceDto] = Field(None, alias='selfhostedInstance')
    storage_rented: Optional[StorageRentedDto] = Field(None, alias='storageRented')

    stderr_log_url: Optional[str] = Field(None, alias='stderrLogUrl')
    stdout_log_url: Optional[str] = Field(None, alias='stdoutLogUrl')
    exit_code: Optional[int] = Field(None, alias='exitCode')
    failure_reason: Optional[str] = Field(None, alias='failureReason')


class TaskViewResponse(TheStageBaseResponse):
    task: Optional[TaskDto] = Field(None, alias='task')
    task_view_url: Optional[str] = Field(None, alias='websiteTaskViewUrl')
    task_output_url: Optional[str] = Field(None, alias='websiteTaskOutputUrl')


class TaskListForSketchPaging(TheStageBaseResponse):
    entities: List[TaskDto] = Field(default_factory=list, alias='entities')
    current_page: Optional[int] = Field(None, alias='currentPage')
    last_page: Optional[bool] = Field(None, alias='lastPage')
    total_pages: Optional[int] = Field(None, alias='totalPages')
    pagination_data: Optional[PaginationData] = Field(None, alias='paginationData')


class TaskListForSketchResponse(TheStageBaseResponse):
    tasks: Optional[TaskListForSketchPaging] = Field(None, alias='tasks')


class TaskGetOutputResponse(TheStageBaseResponse):
    message: Optional[str] = Field(None, alias='message')
    stdout: Optional[str] = Field(None, alias='stdout')
    stderr: Optional[str] = Field(None, alias='stderr')



from typing import Optional

from pydantic import BaseModel, Field, ConfigDict

from thestage.services.clients.thestage_api.dtos.enums.currency_type import CurrencyTypeEnumDto


class PriceDefinitionDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    currency: CurrencyTypeEnumDto = Field(CurrencyTypeEnumDto.UNKNOWN, alias='currency')
    cents_amount_in_unit: Optional[int] = Field(None, alias='centsAmountInUnit')
    cents_value: Optional[str] = Field(None, alias='centsValue')
    decimal_value: Optional[str] = Field(None, alias='decimalValue')



from typing import Optional

from thestage.exceptions.base_exception import BaseAbstractException


class RemoteServerException(BaseAbstractException):

    def __init__(
            self,
            message: str,
            ip_address: Optional[str] = None,
            username: Optional[str] = None,
    ):
        super(RemoteServerException, self).__init__(message=message)
        self._ip_address = ip_address
        self._username = username

    @property
    def ip_address(self):
        return self._ip_address

    @property
    def username(self):
        return self._username



import pathlib
from typing import Optional, List

from thestage.services.clients.thestage_api.dtos.enums.container_status import ContainerBussinessStatusEnumDto, \
    ContainerFrontendStatusEnumDto
from thestage.services.clients.thestage_api.dtos.enums.container_pending_action import ContainerPendingActionEnumDto
from thestage.services.clients.thestage_api.dtos.container_response import DockerContainerDto
from thestage.i18n.translation import __
from thestage.services.container.container_service import ContainerService
from thestage.services.container.mapper.container_mapper import ContainerMapper
from thestage.helpers.logger.app_logger import app_logger
from thestage.controllers.utils_controller import base_check_validation, get_current_directory, base_global_check_validation

import typer


app = typer.Typer(no_args_is_help=True, help=__("Help working with containers"))


@app.command(name='ls', help=__("Show list containers"))
def list_items(
        row: int = typer.Option(
            5,
            '--row',
            '-r',
            help=__("Count row in table"),
            is_eager=False,
        ),
        page: int = typer.Option(
            1,
            '--page',
            '-p',
            help=__("Page number"),
            is_eager=False,
        ),
        statuses: List[ContainerFrontendStatusEnumDto] = typer.Option(
            ["running", "starting"],
            '--status',
            '-s',
            help=__("Status item (all - show all containers)"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        List containers
    """
    path = get_current_directory()
    app_logger.info(f'Start container lists from {path}')

    config, facade = base_global_check_validation(path=path, no_dialog=no_dialog)

    headers = [
        'STATUS',
        'UNIQUE ID',
        'TITLE',
        'INSTANCE UNIQUE ID',
        'SELF-HOSTED UNIQUE ID',
        'DOCKER IMAGE',
    ]

    container_service: ContainerService = facade.get_container_service()

    typer.echo(__(
        "Start show container list with statuses: %statuses% (You can show all statuses set up status=all)",
        placeholders={
            'statuses': ', '.join([item.value for item in statuses])
        }))

    real_statuses: List[str] = container_service.map_container_statuses(config=config, frontend=statuses)

    container_service.print(
        func_get_data=container_service.get_list,
        func_special_params={
            'statuses': real_statuses,
        },
        mapper=ContainerMapper(),
        config=config,
        headers=headers,
        row=row,
        page=page,
        no_dialog=no_dialog,
        max_col_width=[10, 20, 30, 30, 30, 20, 20, 20],
        show_index="never",
    )

    typer.echo(__("List containers done"))
    raise typer.Exit(0)


@app.command(name="info", help=__("Show details container"))
def item_details(
        container_slug: Optional[str] = typer.Option(
            None,
            '--container-uniqueid',
            '-uid',
            help=__("Container unique id"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Show container details
    """
    path = get_current_directory()
    app_logger.info(f'Start container details')

    if not container_slug:
        typer.echo(__('Container id or container slug is required'))
        raise typer.Exit(1)

    config, facade = base_check_validation(path=path, no_dialog=no_dialog)

    container_service: ContainerService = facade.get_container_service()

    container: Optional[DockerContainerDto] = container_service.get_item(
        config=config,
        container_slug=container_slug,
    )

    if not container:
        typer.echo(__("Not found container - %container_item%", {'container_item': str(container_slug) if container_slug else ''}))
        raise typer.Exit(1)

    typer.echo(__('CONTAINER DATA:'))
    typer.echo(__("STATUS: %status%", {'status': str(container.frontend_status.status_translation if container and container.frontend_status else 'UNKNOWN')}))
    typer.echo(__("UNIQUE ID: %slug%", {'slug': str(container.slug)}))
    typer.echo(__("TITLE: %title%", {'title': str(container.title)}))

    if container.instance_rented:
        typer.echo(
            __("INSTANCE RANTED UNIQUE ID: %instance_slug%", {'instance_slug': str(container.instance_rented.slug)})
        )
        typer.echo(
            __("INSTANCE STATUS: %instance_status%",
               {'instance_status': str(container.instance_rented.frontend_status.status_translation if container.instance_rented.frontend_status else 'UNKNOWN')})
        )

    if container.selfhosted_instance:
        typer.echo(
            __("SELFHOSTED INSTANCE UNIQUE ID: %instance_slug%", {'instance_slug': str(container.selfhosted_instance.slug)})
        )
        typer.echo(
            __("SELFHOSTED INSTANCE STATUS: %instance_status%",
               {'instance_status': str(container.selfhosted_instance.frontend_status.status_translation if container.selfhosted_instance.frontend_status else 'UNKNOWN')})
        )

    if container.mappings and (container.mappings.port_mappings or container.mappings.directory_mappings):
        if container.mappings.port_mappings:
            typer.echo(__("CONTAINER PORT MAPPING:"))
            for src, dest in container.mappings.port_mappings.items():
                typer.echo(f"    {src} : {dest}")

        if container.mappings.directory_mappings:
            typer.echo(__("CONTAINER DIRECTORY MAPPING:"))
            for src, dest in container.mappings.directory_mappings.items():
                typer.echo(f"    {src} : {dest}")

    typer.echo(__("Container details done"))
    raise typer.Exit(0)


@app.command(name="connect", help=__("Help connect to container"))
def container_connect(
        container_slug: Optional[str] = typer.Option(
            None,
            '--container-uniqueid',
            '-uid',
            help=__("Container unique id"),
            is_eager=False,
        ),
        username: Optional[str] = typer.Option(
            None,
            '--username',
            '-u',
            help=__("Instance username if use selfhosted"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Show container details
    """
    path = get_current_directory()
    app_logger.info(f'Connect to container')

    if not container_slug:
        typer.echo(__('Container id or container slug is required'))
        raise typer.Exit(1)

    config, facade = base_global_check_validation(path=path, no_dialog=no_dialog)

    container_service: ContainerService = facade.get_container_service()

    container: Optional[DockerContainerDto] = container_service.get_item(
        config=config,
        container_slug=container_slug,
    )

    if container:
        container_service.check_container_status_for_work(
            container=container
        )
        container_service.connect_container(
            config=config,
            container=container,
            no_dialog=no_dialog,
            username_param=username,
        )
    else:
        typer.echo(__("Not found container - %container_item%", {'container_item': container_slug}))

    app_logger.info(f'Stop connect to container')
    raise typer.Exit(0)


@app.command(name="upload", help=__("Help copy file to container"))
def put_file(
        container_slug: Optional[str] = typer.Option(
            None,
            '--container-uniqueid',
            '-s',
            help=__("Container unique id"),
            is_eager=False,
        ),
        source_path: str = typer.Option(
            None,
            '--target-path',
            '-sp',
            help=__("Path to file for copy"),
            is_eager=True,
        ),
        destination_path: Optional[str] = typer.Option(
            None,
            '--destination-path',
            '-dp',
            help=__("Path destination copy"),
            is_eager=False,
        ),
        is_recursive: bool = typer.Option(
            False,
            "--is-recursive",
            "-r",
            help=__("Recursive flag help copy folders"),
            is_eager=False,
        ),
        username: Optional[str] = typer.Option(
            None,
            '--username',
            '-u',
            help=__("Instance username if use selfhosted"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Push file to container
    """
    path = get_current_directory()
    app_logger.info(f'Push file to container')

    if not container_slug:
        typer.echo(__('Container id or container slug is required'))
        raise typer.Exit(1)

    config, facade = base_global_check_validation(path=path, no_dialog=no_dialog)

    container_service: ContainerService = facade.get_container_service()

    container: Optional[DockerContainerDto] = container_service.get_item(
        config=config,
        container_slug=container_slug,
    )

    if container:
        container_service.check_container_status_for_work(
            container=container
        )

        container_service.put_file_to_container(
            container=container,
            src_path=source_path,
            destination_path=destination_path,
            is_folder=is_recursive,
            no_dialog=no_dialog,
            username_param=username,
        )
    else:
        typer.echo(__("Not found container - %container_item%", {'container_item': container_slug}))

    app_logger.info(f'End send files to container')
    raise typer.Exit(0)


@app.command(name="download", help=__("Help copy file from container"))
def download_file(
        container_slug: Optional[str] = typer.Option(
            None,
            '--container-uniqueid',
            '-s',
            help=__("Container unique id"),
            is_eager=False,
        ),
        source_path: str = typer.Option(
            None,
            '--target-path',
            '-sp',
            help=__("Path to file for copy"),
            is_eager=True,
        ),
        destination_path: Optional[str] = typer.Option(
            None,
            '--destination-path',
            '-dp',
            help=__("Path destination copy"),
            is_eager=False,
        ),
        is_recursive: bool = typer.Option(
            False,
            "--is-recursive",
            "-r",
            help=__("Recursive flag help copy folders"),
            is_eager=False,
        ),
        username: Optional[str] = typer.Option(
            None,
            '--username',
            '-u',
            help=__("Instance username if use selfhosted"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Download file from container
    """
    path = get_current_directory()
    app_logger.info(f'Download file from container')

    if not container_slug:
        typer.echo(__('Container id or container slug is required'))
        raise typer.Exit(1)

    config, facade = base_global_check_validation(path=path, no_dialog=no_dialog)

    container_service: ContainerService = facade.get_container_service()

    container: Optional[DockerContainerDto] = container_service.get_item(
        config=config,
        container_slug=container_slug,
    )

    if container:
        container_service.check_container_status_for_work(
            container=container
        )

        container_service.get_file_from_container(
            container=container,
            src_path=source_path,
            destination_path=destination_path,
            is_folder=is_recursive,
            no_dialog=no_dialog,
            username_param=username,
        )
    else:
        typer.echo(__("Not found container - %container_item%", {'container_item': container_slug}))

    app_logger.info(f'End download files from container')
    raise typer.Exit(0)


@app.command(name="start", help=__("Help start container"))
def item_start(
        container_slug: Optional[str] = typer.Option(
            None,
            '--container-uniqueid',
            '-s',
            help=__("Container unique id"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Start container
    """
    path = get_current_directory()
    app_logger.info(f'Start container')

    if not container_slug:
        typer.echo(__('Container id or container slug is required'))
        raise typer.Exit(1)

    config, facade = base_global_check_validation(path=path, no_dialog=no_dialog)

    container_service: ContainerService = facade.get_container_service()

    container: Optional[DockerContainerDto] = container_service.get_item(
        config=config,
        container_slug=container_slug,
    )

    if container:
        container_service.check_container_status_for_start(
            container=container
        )
        result = container_service.change_container_status(
            config=config,
            container=container,
            action=ContainerPendingActionEnumDto.START
        )

        if result:
            typer.echo(__('Container will be started'))
        else:
            typer.echo(__('Something went wrong on server, please change late'))
    else:
        typer.echo(__("Not found container - %container_item%", {'container_item': container_slug}))

    app_logger.info(f'End start container')
    raise typer.Exit(0)


@app.command(name="stop", help=__("Help stop container"))
def item_stop(
        container_slug: Optional[str] = typer.Option(
            None,
            '--container-uniqueid',
            '-s',
            help=__("Container unique id"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Stop container
    """
    path = get_current_directory()
    app_logger.info(f'Stop container')

    if not container_slug:
        typer.echo(__('Container id or container slug is required'))
        raise typer.Exit(1)

    config, facade = base_global_check_validation(path=path, no_dialog=no_dialog)

    container_service: ContainerService = facade.get_container_service()

    container: Optional[DockerContainerDto] = container_service.get_item(
        config=config,
        container_slug=container_slug,
    )

    if container:
        container_service.check_container_status_for_stop(
            container=container
        )
        result = container_service.change_container_status(
            config=config,
            container=container,
            action=ContainerPendingActionEnumDto.STOP
        )

        if result:
            typer.echo(__('Container will be stoped'))
        else:
            typer.echo(__('Something went wrong on server, please change late'))
    else:
        typer.echo(__("Not found container - %container_item%", {'container_item': container_slug}))

    app_logger.info(f'End stop container')
    raise typer.Exit(0)



from typing import Optional

from python_gettext_translations.translations import translate

from thestage.config import THESTAGE_LOCAL_LANGUAGE


def __(original: str, placeholders: Optional[dict] = None) -> str:
    return translate(THESTAGE_LOCAL_LANGUAGE, original, placeholders)



import pathlib
from typing import Optional, Dict, Tuple

from thestage_core.entities.config_entity import ConfigEntity

from thestage.entities.enums.config_type import ConfigType
from thestage.helpers.error_handler import error_handler
from thestage.services.service_facade import ServiceFacade
from thestage.services.config_provider.config_provider import ConfigProvider


def get_current_directory() -> pathlib.Path:
    return pathlib.Path.cwd()


@error_handler()
def base_check_validation(
        path: str,
        working_directory: Optional[str] = None,
        no_dialog: bool = False,
) -> Tuple[ConfigEntity, ServiceFacade]:
    config_provider = ConfigProvider(project_path=path if not working_directory else working_directory)
    facade = ServiceFacade(config_provider=config_provider)
    config: ConfigEntity = config_provider.get_full_config()

    if no_dialog:
        config.runtime.config_type = ConfigType.LOCAL

    validation_service = facade.get_validation_service()
    validation_service.check_token(config=config, no_dialog=no_dialog)
    config_provider.save_full_config(config=config, config_type=config.runtime.config_type)
    return config, facade


@error_handler()
def base_global_check_validation(
        path: str,
        working_directory: Optional[str] = None,
        no_dialog: bool = False,
) -> Tuple[ConfigEntity, ServiceFacade]:
    config_provider = ConfigProvider(
        project_path=path if not working_directory else working_directory,
        auto_create=True,
        only_global=True,
    )
    facade = ServiceFacade(config_provider=config_provider)
    config: ConfigEntity = config_provider.get_full_config(only_global=True)

    validation_service = facade.get_validation_service()
    validation_service.check_token(config=config, no_dialog=no_dialog, only_global=True)
    config_provider.save_full_config(
        config=config,
        config_type=config.runtime.config_type
    )
    return config, facade
